package net.serice.checksum;

import java.util.LinkedList;

public class Flags {

    private String help = ""
+ "\n"
+ "Usage: java -jar checksum.jar [options] [files]\n"
+ "\n"
+ "Options:\n"
+ "\n"
+ "                -md5 : Use MD5 hash function.  This is the default.\n"
+ "               -sha1 : Use SHA-1 hash function.\n"
+ "             -sha256 : Use SHA-256 hash function.\n"
+ "             -sha384 : Use SHA-384 hash function.\n"
+ "             -sha512 : Use SHA-512 hash function.\n"
+ "                  -- : Disables option parsing; only files remain.\n"
+ "                  -h : Print help.\n"
+ "       -f <function> : If one of the above hash functions is not what you\n"
+ "                       want, you can specify the an alternative here.\n"
+ "                       The hash functions above are the ones available\n"
+ "                       in the SUN provider which is the default provider\n"
+ "                       in an unaltered installation.  If you use this\n"
+ "                       option with any of the above hash functions, the\n"
+ "                       last one you specify is the one that is used.\n"
+ "       -p <provider> : Use <provider> as the security provider.  The\n"
+ "                       default is the first provider in\n"
+ "                       jre/lib/security/java.security that provides the\n"
+ "                       requested hash function.  Do not use this option\n"
+ "                       with the -r option.\n"
+ "           -r <root> : Use <root> as the root class to load for your\n"
+ "                       provider.  Do not use this option with the -p\n"
+ "                       option.\n"
+ "                  -v : Print version.\n"
+ "\n"
+ "Files:\n"
+ "\n"
+ "        If no files are specified, standard input is used.\n"
+ "\n"
+ "Examples:\n"
+ "\n"
+ "  1) java -jar checksum.jar\n"
+ "\n"
+ "  2) java -jar checksum.jar -sha1\n"
+ "\n"
+ "  3) java -jar checksum.jar -sha1 -p CryptixCrypto\n"
+ "\n"
+ "  4) java -jar checksum.jar -f ripemd320 -p bc\n"
+ "\n"
+ "  5) export CLASSPATH=checksum.jar:cryptix-jce-provider.jar\n"
+ "     java net.serice.checksum.Main \\\n"
+ "          -f tiger -r cryptix.jce.provider.CryptixCrypto\n"
+ "\n"
+ "  6) export CLASSPATH=checksum.jar:bcprov-jdk14-124.jar\n"
+ "     java net.serice.checksum.Main \\\n"
+ "          -f tiger -r org.bouncycastle.jce.provider.BouncyCastleProvider\n"
;

    private boolean dashDashSeen;
    private LinkedList files = new LinkedList();
    private boolean helpRequested;
    private String hash = "MD5";
    private String provider;
    private String providerRoot;
    private boolean versionRequested;

    public Flags(String[] args) {
        for (int i = 0 ; i < args.length ; ++i) {
            String arg = args[i];

            // Check for options?
            if (!dashDashSeen && (arg.charAt(0) == '-')) {

                // Hash
                if (   arg.equalsIgnoreCase("-md5")
                    || arg.equalsIgnoreCase("--md5"))
                {
                    hash = "MD5";
                    continue;
                }
                if (   arg.equalsIgnoreCase("-sha1")
                    || arg.equalsIgnoreCase("--sha1"))
                {
                    hash = "SHA-1";
                    continue;
                }
                if (   arg.equalsIgnoreCase("-sha256")
                    || arg.equalsIgnoreCase("--sha256"))
                {
                    hash = "SHA-256";
                    continue;
                }
                if (   arg.equalsIgnoreCase("-sha384")
                    || arg.equalsIgnoreCase("--sha384"))
                {
                    hash = "SHA-384";
                    continue;
                }
                if (   arg.equalsIgnoreCase("-sha512")
                    || arg.equalsIgnoreCase("--sha512"))
                {
                    hash = "SHA-512";
                    continue;
                }

                // Function
                if (   arg.equalsIgnoreCase("-f")
                    || arg.equalsIgnoreCase("-function")
                    || arg.equalsIgnoreCase("--function"))
                {
                    if (i + 1 < args.length) {
                        hash = args[++i];
                    } else {
                        failAtIndex(arg);
                        break;
                    }
                    continue;
                }

                // Help
                if (   arg.equalsIgnoreCase("-h")
                    || arg.equalsIgnoreCase("-help")
                    || arg.equalsIgnoreCase("--help"))
                {
                    helpRequested = true;
                    continue;
                }

                // Provider
                if (   arg.equalsIgnoreCase("-p")
                    || arg.equalsIgnoreCase("-provider")
                    || arg.equalsIgnoreCase("--provider"))
                {
                    if (i + 1 < args.length) {
                        provider = args[++i];
                    } else {
                        failAtIndex(arg);
                        break;
                    }
                    continue;
                }

                // Root
                if (   arg.equalsIgnoreCase("-r")
                    || arg.equalsIgnoreCase("-root")
                    || arg.equalsIgnoreCase("--root"))
                {
                    if (i + 1 < args.length) {
                        providerRoot = args[++i];
                    } else {
                        failAtIndex(arg);
                        break;
                    }
                    continue;
                }

                // Version
                if (   arg.equalsIgnoreCase("-v")
                    || arg.equalsIgnoreCase("-version")
                    || arg.equalsIgnoreCase("--version"))
                {
                    versionRequested = true;
                    continue;
                }

                // --
                if (arg.equals("--")) {
                    dashDashSeen = true;
                    continue;
                }

                failAtIndex(arg);
                break;
            }

            // Files
            files.add(arg);
        }

        // Verify the flags are consistent.
        if ((provider != null) && (providerRoot != null)) {
            String msg =   "Invalid flag combination: -p and -r are mutually "
                         + "exclusive.";
            throw new RuntimeException(msg);
        }
    }

    private void failAtIndex(String arg) {
        throw new RuntimeException("Invalid flag: \"" + arg + "\"");
    }

    public LinkedList getFiles() {
        return files;
    }

    public String getHelp() {
        return help;
    }

    public boolean isHelpRequested() {
        return helpRequested;
    }

    public String getHash() {
        return hash;
    }

    public String getProvider() {
        return provider;
    }

    public String getProviderRoot() {
        return providerRoot;
    }

    public boolean isVersionRequested() {
        return versionRequested;
    }

}
