package net.serice.checksum;

import java.io.*;
import java.math.BigInteger;
import java.security.*;
import java.util.*;

public class Main {

    private static final int BUFSIZE = 16384;

    private static Flags flags;

    public static void main(String[] args) {

        int rv = 0;

        done:
        try {

            // Parse command line.
            flags = new Flags(args);
            if (flags.isHelpRequested()) {
                System.out.println(flags.getHelp());
                break done;
            }
            if (flags.isVersionRequested()) {
                System.out.println("checksum v1.0.2");
                break done;
            }

            // Get the MessageDigest.
            MessageDigest md = getMessageDigest();

            // Process each file in turn.
            LinkedList files = flags.getFiles();
            if (files.size() == 0) {
                // No files were specified on the command line.  Use stdin.
                processFile(System.in, "-", md);
            } else {
                // Use the files specified on the command line.
                Iterator i = files.listIterator();
                while (i.hasNext()) {
                    String fname = (String)i.next();
                    InputStream istrm = new FileInputStream(fname);
                    processFile(istrm, fname, md);
                    istrm.close();
                }
            }

        } catch (Exception e) {
            String msg = e.getLocalizedMessage();
            if (msg != null) {
                System.err.println("***");
                System.err.println("*** checksum: Error: " + msg);
                System.err.println("***");
            } else {
                e.printStackTrace();
            }
            rv = 1;
        }

        done:
        System.exit(rv);

    }

    private static MessageDigest getMessageDigest() throws Exception {
        MessageDigest rv;
        String h = flags.getHash();
        String p = flags.getProvider();
        String r = flags.getProviderRoot();
        if ((p == null) && (r == null)) {
            // Load provider BY ALGORITHM using default providers in
            // jre/lib/security/java.security.
            rv = MessageDigest.getInstance(h);
        } else if (p != null) {
            // Load provider BY NAME including using external
            // knowledge of the root class for BouncyCastle and
            // Cryptix providers.
            Provider pr = Util.getProvider(p);
            if (pr == null) {
                String msg = "Unable to get provider by name \"" + p + "\".";
                throw new RuntimeException(msg);
            }
            rv = MessageDigest.getInstance(h, pr);
        } else if (r != null) {
            // Load provider BY ROOT CLASS.
            Provider pr = Util.loadProviderDynamically(r);
            if (pr == null) {
                String msg =
                 "Unable to load provider by root class " + "\"" + r + "\".";
                throw new RuntimeException(msg);
            }
            rv = MessageDigest.getInstance(h, pr);
        } else {
            String msg = "Internal Logic failure finding provider.";
            throw new RuntimeException(msg);
        }
        return rv;
    }

    private static void processFile(InputStream istrm,
                                    String fname,
                                    MessageDigest md) throws Exception
    {
        int rcount;
        byte[] buf = new byte[BUFSIZE];
        md.reset();
        DigestInputStream f = new DigestInputStream(istrm, md);
        for ( ;; ) {
            if (f.read(buf, 0, BUFSIZE) <= 0) {
                break;
            }
        }
        byte[] mdBytes = md.digest();
        BigInteger n = new BigInteger(1, mdBytes);
        String hex = n.toString(16);
        // We may need to pad with leading zeros to make sure the
        // checksum prints the right number of hexadecimal digits.
        // You multiply by two because there are two hex digits per
        // byte.
        int i_max = md.getDigestLength() * 2 - hex.length();
        for (int i = 0 ; i < i_max ; ++i) {
            System.out.print("0");
        }
        System.out.println( hex + "  " + fname);
    }

}
